%define anolis_release 1
# We can skip tests
%bcond_without testsuite

%if %{without testsuite}
%global tcl 0
%global guile 0
%global lualang 0
%global perllang 0
%global phplang 0
%global rubylang 0
%global python3lang 0
%global golang 0
%global octave 0
%global Rlang 0
%global javalang 0
%endif

%{!?tcl:%global tcl 1}
%{!?lualang:%global lualang 1}
%{!?perllang:%global perllang 1}
%{!?phplang:%global phplang 1}
%{!?rubylang:%global rubylang 1}
%{!?python3lang:%global python3lang 1}

%{!?guile:%global guile 1}
%{!?octave:%global octave 0}
%{!?Rlang:%global Rlang 0}
%bcond_without build_ccache_swig

%{!?javalang:%global javalang 1}

%{!?golang:%global golang 1}

Summary: Connects C/C++/Objective C to some high-level programming languages
Name:    swig
Version: 4.3.0
Release: %{anolis_release}%{?dist}
License: GPL-3.0-or-later AND BSD-3-Clause
URL:     http://swig.sourceforge.net/
Source0: http://downloads.sourceforge.net/project/swig/swig/swig-%{version}/swig-%{version}.tar.gz
# Define the part of man page sections
Source1: description.h2m
%if %{with build_ccache_swig}
Source2: description-ccache.h2m
Source3: ccache-swig.sh
Source4: ccache-swig.csh
%endif

# Affect version: 4.3.0
# https://github.com/swig/swig/commit/4e315cdd7c9072fd33ac1df3d208a0990b8231c3
Patch0001:  swig-4.3.0-Fix-precedence-of-casts.patch

BuildRequires: coreutils
BuildRequires: findutils
BuildRequires: make
BuildRequires: perl-interpreter, pcre2-devel
BuildRequires: python%{python3_pkgversion}-devel
BuildRequires: autoconf, automake, gawk, dos2unix
BuildRequires: gcc-c++
BuildRequires: help2man
BuildRequires: sed
BuildRequires: perl-devel
BuildRequires: perl(base)
BuildRequires: perl(Config)
BuildRequires: perl(Devel::Peek)
BuildRequires: perl(ExtUtils::MakeMaker)
BuildRequires: perl(fields)
BuildRequires: perl(Math::BigInt)
BuildRequires: perl(strict)
BuildRequires: perl(Test::More)
BuildRequires: perl(vars)
BuildRequires: perl(warnings)
BuildRequires: boost-devel
BuildRequires: gmp-devel
BuildRequires: gc-devel
# Need when Source/CParse/parser.y is patched
BuildRequires: bison
%if %{tcl}
BuildRequires: tcl-devel
%endif
%if %{guile}
BuildRequires: guile-devel
%endif
%if %{octave}
BuildRequires: octave-devel
%endif
%if %{golang}
BuildRequires: golang
BuildRequires: golang-bin
%ifnarch loongarch64
BuildRequires: golang-shared
%endif
BuildRequires: golang-src
%endif
%if %{lualang}
BuildRequires: lua-devel
%endif
%if %{rubylang}
BuildRequires: ruby-devel
%endif
%if %{Rlang}
BuildRequires: R-devel
%endif
%if %{javalang}
BuildRequires: java, java-devel
%endif
%if %{phplang}
BuildRequires: php, php-devel
%endif

%description
Simplified Wrapper and Interface Generator (SWIG) is a software
development tool for connecting C, C++ and Objective C programs with a
variety of high-level programming languages. SWIG is used with different
types of target languages including common scripting languages such as
Javascript, Perl, PHP, Python, Tcl and Ruby. The list of supported
languages also includes non-scripting languages such as C#, D, Go language,
Java including Android, Lua, OCaml, Octave, Scilab and R. Also several
interpreted and compiled Scheme implementations (Guile, MzScheme/Racket)
are supported. SWIG is most commonly used to create high-level interpreted
or compiled programming environments, user interfaces, and as a tool for
testing and prototyping C/C++ software.

%if %{with build_ccache_swig}
%package -n ccache-swig
Summary:   Fast compiler cache
License:   GPL-2.0-or-later
Requires:  swig
Conflicts: swig < 3.0.8-2

%description -n ccache-swig
ccache-swig is a compiler cache. It speeds up re-compilation of C/C++/SWIG
code by caching previous compiles and detecting when the same compile is
being done again. ccache-swig is ccache plus support for SWIG.
%endif

%package doc
Summary:   Documentation files for SWIG
License:   BSD-3-Clause
BuildArch: noarch

%description doc
This package contains documentation for SWIG and useful examples

%package gdb
Summary:   Commands for easier debugging of SWIG
License:   BSD-3-Clause
Requires:  swig

%description gdb
This package contains file with commands for easier debugging of SWIG
in gdb.

%prep
%autosetup -p1

for all in CHANGES README; do
    iconv -f ISO88591 -t UTF8 < $all > $all.new
    touch -r $all $all.new
    mv -f $all.new $all
done

%build
./autogen.sh

# Disable maximum compile warnings when octave is supported, because Octave
# code produces lots of the warnings demanded by strict ISO C and ISO C++.
# It causes that log had more then 600M.
# AC_CHECK_PROGS requires just the name, so use for configure
#   --with-python3=python3 --with-2to3=2to3
%configure \
  --without-ocaml \
%if %{python3lang}
  --with-python3=python3 \
  --with-2to3=2to3 \
%else
  --without-python3 \
%endif
%if %{phplang}
  --with-php \
%else
  --without-php \
%endif
%if ! %{perllang}
  --without-perl5 \
%endif
%if ! %{tcl}
  --without-tcl \
%endif
%if ! %{javalang}
  --without-java \
%endif
%if ! %{Rlang}
  --without-r \
%endif
%if ! %{golang}
  --without-go \
%endif
%if ! %{guile}
  --without-guile \
%endif
%if %{octave}
  --with-octave=%{_bindir}/octave \
  --without-maximum-compile-warnings \
%endif
%if %{without build_ccache_swig}
  --disable-ccache \
%endif
;
%{make_build}

%if %{with testsuite}
# Test suite
make check PY3=1
%endif

%install
# Remove all arch dependent files in Examples/ created during tests
make clean-examples

pushd Examples/
# Remove all arch dependent files in Examples/
find -type f -name 'Makefile.in' -delete -print

# We don't want to ship files below.
rm -rf test-suite
find -type f -name '*.dsp' -delete -print
find -type f -name '*.dsw' -delete -print

# Convert files to UNIX format
for all in `find -type f`; do
    dos2unix -k $all
    chmod -x $all
done
popd

%{make_install}

#################################################
# Use help output for generating of man page swig
echo "Options:" >help_swig
%{buildroot}%{_bindir}/swig --help >>help_swig

# Update the output to be correctly formatted be help2man
sed -i -e 's/^\(\s\+-[^-]\+\)- \(.*\)$/\1 \2/' help_swig
sed -i -e 's/^\(\s\+-\w\+-[^-]*\)- \(.*\)$/\1 \2/' help_swig

# Generate a helper script that will be used by help2man
cat >h2m_helper_swig <<'EOF'
#!/bin/bash
[ "$1" == "--version" ] && echo "" || cat help_swig
EOF
chmod a+x h2m_helper_swig

# Generate man page
help2man -N --section 1 ./h2m_helper_swig --include %{SOURCE1} -o %{name}.1

%if %{with build_ccache_swig}
########################################################
# Use help output for generating of man page ccache-swig
%{buildroot}%{_bindir}/ccache-swig -h >>help_ccache

# Update the output to be correctly formatted be help2man
sed -i -e '/compiler cache/ d' help_ccache
sed -i -e '/Copyright/ d' help_ccache
sed -i -e 's/^Usage:/[synopsis]/' help_ccache
sed -i -e 's/^Options:/[options]/' help_ccache
sed -i -e 's/^\s\+/ /' help_ccache
sed -i -e 's/^\(-[^- ] <\w\+>\s\+\) \(\w.\+\)$/ \1 \2/' help_ccache
sed -i -e 's/^\(-[^- ]\s\+\) \(\w.\+\)$/ \1 \2/' help_ccache

# Generate a helper script that will be used by help2man
cat >h2m_helper_ccache <<'EOF'
#!/bin/bash
[ "$1" == "--version" ] && echo ""
[ "$1" == "--help" ] && echo "" || echo ""
EOF
chmod a+x h2m_helper_ccache

cat %{SOURCE2} >>help_ccache
sed -i -e 's#@DOCDIR@#%{_docdir}#' help_ccache

# Generate man page
help2man -N --section 1 ./h2m_helper_ccache --include help_ccache -o ccache-swig.1
%endif

# Add man page for swig to repository
mkdir -p %{buildroot}%{_mandir}/man1/
install -p -m 0644 %{name}.1 %{buildroot}%{_mandir}/man1/
%if %{with build_ccache_swig}
install -p -m 0644 ccache-swig.1 %{buildroot}%{_mandir}/man1/
%endif

# Quiet some rpmlint complaints - remove empty file
rm -f %{buildroot}%{_datadir}/%name/%{version}/octave/std_carray.i

%if %{with build_ccache_swig}
# Enable ccache-swig by default
mkdir -p %{buildroot}%{_sysconfdir}/profile.d/
install -dm 755 %{buildroot}%{_sysconfdir}/profile.d
install -pm 644 %{SOURCE3} %{SOURCE4} %{buildroot}%{_sysconfdir}/profile.d
%endif

# Add swig.gdb sub-package gdb
mkdir -p %{buildroot}%{_datadir}/%{name}/gdb
install -pm 644 Tools/swig.gdb %{buildroot}%{_datadir}/%{name}/gdb

%files
%license COPYRIGHT LICENSE LICENSE-GPL LICENSE-UNIVERSITIES
%{_bindir}/%{name}
%{_datadir}/%{name}
%exclude %{_datadir}/%{name}/gdb
%{_mandir}/man1/swig.1*

%if %{with build_ccache_swig}
%files -n ccache-swig
%{_bindir}/ccache-swig
%config(noreplace) %{_sysconfdir}/profile.d/ccache-swig.*sh
%{_mandir}/man1/ccache-swig.1*
%endif

%files doc
%license LICENSE LICENSE-GPL LICENSE-UNIVERSITIES
%doc Doc Examples COPYRIGHT
%doc ANNOUNCE CHANGES CHANGES.current
%doc README TODO

%files gdb
%{_datadir}/%{name}/gdb

%changelog
* Tue Feb 18 2025 gaochang <gc-taifu@linux.alibaba.com> - 4.3.0-1
- Update to 4.3.0
- Remove patches because bugs are not exist in new version.
- Add patches to fix the precedence of casts is lower than expected.
  [Patches cherry-picked from c10s. Original changelog:
  - Fix precedence of casts. (4.3.0-3)]

* Thu Oct 12 2023 Wenlong Zhang<zhangwenlong@loongson.cn> - 4.1.1-3
- fix build error for swig
 
* Mon Jan 30 2023 happy_orange <songnannan@linux.alibaba.com> - 4.1.1-2
- Remove the define of PY_SSIZE_T_CLEAN

* Wed Jan 18 2023 Funda Wang <fundawang@yeah.net> - 4.1.1-1
- Import package for anolis 23
